@@ -16,12 +16,12 @@
 # IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 # MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 # EVENT SHALL CHRISTOPH RELLER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
-# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
-# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
-# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 # The views and conclusions contained in the software and documentation are
 # those of the authors and should not be interpreted as representing official
@@ -38,7 +38,7 @@
     Author: Christoph Reller <christoph.reller@gmail.com>
 """
 
-__version__ = '0.4.13'
+__version__ = '0.4.14'
 
 import contextlib
 import tempfile
@@ -65,7 +65,7 @@ from docutils.statemachine import ViewList
 from sphinx.errors import SphinxError
 try:
     from sphinx.util.osutil import ensuredir
-except:
+except ImportError:
     from sphinx.util import ensuredir
 
 from glob import glob
@@ -129,6 +129,41 @@ class tikz(nodes.Part, nodes.Element):
     pass
 
 
+DOC_HEAD = r'''
+\documentclass[12pt,tikz]{standalone}
+\usepackage[utf8]{inputenc}
+\usepackage{amsmath}
+\usepackage{pgfplots}
+\usetikzlibrary{%s}
+\pagestyle{empty}
+'''
+
+DOC_BODY = r'''
+\begin{document}
+%s
+\end{document}
+'''
+
+OUT_EXTENSION = {
+    'GhostScript': 'png',
+    'ImageMagick': 'png',
+    'Netpbm': 'png',
+    'pdf2svg': 'svg',
+    }
+
+LATEX_ALIGN = {
+    'center': 'centering',
+    'left': 'raggedright',
+    'right': 'raggedleft',
+    }
+
+LATEX_ALIGN_ENV = {
+    'center': 'center',
+    'left': 'flushleft',
+    'right': 'flushright',
+    }
+
+
 class TikzDirective(Directive):
     has_content = True
     required_arguments = 0
@@ -196,46 +231,12 @@ class TikzDirective(Directive):
 
         return [node]
 
-DOC_HEAD = r'''
-\documentclass[12pt,tikz]{standalone}
-\usepackage[utf8]{inputenc}
-\usepackage{amsmath}
-\usepackage{pgfplots}
-\usetikzlibrary{%s}
-\pagestyle{empty}
-'''
-
-DOC_BODY = r'''
-\begin{document}
-%s
-\end{document}
-'''
-
-OUT_EXTENSION = {
-    'GhostScript': 'png',
-    'ImageMagick': 'png',
-    'Netpbm': 'png',
-    'pdf2svg': 'svg',
-    }
-
-LATEX_ALIGN = {
-    'center': 'centering',
-    'left': 'raggedright',
-    'right': 'raggedleft',
-    }
-
-LATEX_ALIGN_ENV = {
-    'center': 'center',
-    'left': 'flushleft',
-    'right': 'flushright',
-    }
-
 
 def cleanup_tikzcode(self, node):
     tikz = node['tikz']
     tikz = tikz.replace('\r\n', '\n')
-    tikz = re.sub('^\s*%.*$\n', '', tikz, 0, re.MULTILINE)
-    tikz = re.sub('^\s*$\n', '', tikz, 0, re.MULTILINE)
+    tikz = re.sub(r'^\s*%.*$\n', '', tikz, 0, re.MULTILINE)
+    tikz = re.sub(r'^\s*$\n', '', tikz, 0, re.MULTILINE)
     if not tikz.startswith('\\begin{tikz'):
         tikz = '\\begin{tikzpicture}\n' + tikz + '\n\\end{tikzpicture}'
     if 'stringsubst' in node:
@@ -244,11 +245,11 @@ def cleanup_tikzcode(self, node):
 
 
 def render_tikz(self, node, libs='', stringsubst=False):
+    config = self.builder.config
     # must use unique filenames for all tmpfiles to support sphinx -j
     tikz = cleanup_tikzcode(self, node)
     shasum = sha(tikz.encode('utf-8')).hexdigest()
-    fname = 'tikz-%s.%s' % (shasum,
-                            OUT_EXTENSION[self.builder.config.tikz_proc_suite])
+    fname = 'tikz-%s.%s' % (shasum, OUT_EXTENSION[config.tikz_proc_suite])
     relfn = posixpath.join(self.builder.imgpath, fname)
     outfn = path.join(self.builder.outdir, '_images', fname)
 
@@ -261,7 +262,8 @@ def render_tikz(self, node, libs='', stringsubst=False):
     ensuredir(path.dirname(outfn))
 
     latex = DOC_HEAD % libs
-    latex += self.builder.config.tikz_latex_preamble
+    latex += self.builder._tikz_preamble
+    latex += config.tikz_latex_preamble
     latex += DOC_BODY % tikz
     latex = latex.encode('utf-8')
 
@@ -271,20 +273,20 @@ def render_tikz(self, node, libs='', stringsubst=False):
         tf.write(latex)
         tf.close()
 
-        system([self.builder.config.latex_engine, '--interaction=nonstopmode',
+        system([config.latex_engine, '--interaction=nonstopmode',
                 'tikz-%s.tex' % shasum],
                self.builder)
 
-        resolution = str(self.builder.config.tikz_resolution)
+        resolution = str(config.tikz_resolution)
 
-        if self.builder.config.tikz_proc_suite in ['ImageMagick', 'Netpbm']:
+        if config.tikz_proc_suite in ['ImageMagick', 'Netpbm']:
 
-            system(['pdftoppm', '-r', resolution, 'tikz-%s.pdf' % shasum, 'tikz-%s' %
-                    shasum], self.builder)
+            system(['pdftoppm', '-r', resolution, 'tikz-%s.pdf' % shasum,
+                    'tikz-%s' % shasum], self.builder)
             ppmfilename = glob('tikz-%s*.ppm' % shasum)[0]
 
-            if self.builder.config.tikz_proc_suite == "ImageMagick":
-                if self.builder.config.tikz_transparent:
+            if config.tikz_proc_suite == "ImageMagick":
+                if config.tikz_transparent:
                     convert_args = ['-fuzz', '2%', '-transparent', 'white']
                 else:
                     convert_args = []
@@ -292,24 +294,26 @@ def render_tikz(self, node, libs='', stringsubst=False):
                 system([which('convert'), '-trim'] + convert_args +
                        [ppmfilename, outfn], self.builder)
 
-            elif self.builder.config.tikz_proc_suite == "Netpbm":
-                if self.builder.config.tikz_transparent:
+            elif config.tikz_proc_suite == "Netpbm":
+                if config.tikz_transparent:
                     pnm_args = ['-transparent', 'rgb:ff/ff/ff']
                 else:
                     pnm_args = []
                 system(['pnmtopng'] + pnm_args + [ppmfilename], self.builder,
                        outfile=outfn)
 
-        elif self.builder.config.tikz_proc_suite == "GhostScript":
-            ghostscript = which('ghostscript') or which('gs') or which('gswin64')
-            if self.builder.config.tikz_transparent:
+        elif config.tikz_proc_suite == "GhostScript":
+            ghostscript = which('ghostscript') or which('gs') or \
+                which('gswin64')
+            if config.tikz_transparent:
                 device = "pngalpha"
             else:
                 device = "png256"
-            system([ghostscript, '-dBATCH', '-dNOPAUSE', '-sDEVICE=%s' % device,
-                    '-sOutputFile=%s' % outfn, '-r' + resolution + 'x' + resolution,
+            system([ghostscript, '-dBATCH', '-dNOPAUSE',
+                    '-sDEVICE=%s' % device, '-sOutputFile=%s' % outfn,
+                    '-r' + resolution + 'x' + resolution,
                     '-f', 'tikz-%s.pdf' % shasum], self.builder)
-        elif self.builder.config.tikz_proc_suite == "pdf2svg":
+        elif config.tikz_proc_suite == "pdf2svg":
             system(['pdf2svg', 'tikz-%s.pdf' % shasum, outfn], self.builder)
         else:
             self.builder._tikz_warned = True
@@ -349,7 +353,9 @@ def html_visit_tikz(self, node):
         scale = ''
         if node['xscale']:
             scale = 'width="%s%%"' % (node['xscale'])
-        self.body.append(self.starttag(node, 'div', CLASS='figure', STYLE='text-align: %s' % self.encode(node['align']).strip()))
+        style = 'text-align: %s' % self.encode(node['align'])
+        tag = self.starttag(node, 'div', CLASS='figure', STYLE=style)
+        self.body.append(tag.strip())
         self.body.append('<p>')
         self.body.append('<img %s src="%s" alt="%s" /></p>\n' %
                          (scale, fname, self.encode(node['alt']).strip()))
@@ -385,12 +391,12 @@ def latex_visit_tikz(self, node):
 
     # If scaling option is set, enclose in resizebox
     scale_start = r""
-    scale_end   = r""
+    scale_end = r""
     scale = 0
     if node['xscale']:
         scale = int(node['xscale']) * 0.01
-        scale_start= r"\resizebox{" + str(scale) + r"\columnwidth}{!}{"
-        scale_end  = r"}"
+        scale_start = r"\resizebox{" + str(scale) + r"\columnwidth}{!}{"
+        scale_end = r"}"
     tikz = scale_start + tikz + scale_end
 
     # Have a caption: enclose in a figure environment.
@@ -429,6 +435,17 @@ def cleanup_tempdir(app, exc):
 
 def builder_inited(app):
     app.builder._tikz_tempdir = tempfile.mkdtemp()
+    app.builder._tikz_preamble = ''
+    extgfxpath = app.builder.config.tikz_includegraphics_path
+    if not extgfxpath == '':
+        if isinstance(extgfxpath, str):
+            extgfxpath = [extgfxpath]
+        app.builder._tikz_preamble = "\\graphicspath{"
+        for s in extgfxpath:
+            if s[-1] != "/":
+                s += "/"
+            app.builder._tikz_preamble += '{"%s/%s"}' % (app.srcdir, s)
+        app.builder._tikz_preamble += "}\n"
 
     if app.builder.name == "latex":
         sty_path = os.path.join(app.builder._tikz_tempdir,
@@ -441,8 +458,9 @@ def builder_inited(app):
         tikzlibs = app.builder.config.tikz_tikzlibraries
         tikzlibs = tikzlibs.replace(' ', '')
         tikzlibs = tikzlibs.replace('\t', '')
-        tikzlibs = tikzlibs.strip(', ') + "\n"
-        sty.write(r"\usetikzlibrary{%s}" % tikzlibs)
+        tikzlibs = tikzlibs.strip(', ')
+        sty.write("\\usetikzlibrary{%s}\n" % tikzlibs)
+        sty.write(app.builder._tikz_preamble)
         sty.write(app.builder.config.tikz_latex_preamble + "\n")
         sty.close()
 
@@ -473,16 +491,17 @@ def which(program):
 
 def setup(app):
     app.add_enumerable_node(tikz, 'figure',
-                 html=(html_visit_tikz, html_depart_tikz),
-                 latex=(latex_visit_tikz, latex_depart_tikz))
+                            html=(html_visit_tikz, html_depart_tikz),
+                            latex=(latex_visit_tikz, latex_depart_tikz))
     app.add_node(tikzinline,
                  html=(html_visit_tikzinline, depart_tikzinline),
                  latex=(latex_visit_tikzinline, depart_tikzinline))
     app.add_role('tikz', tikz_role)
     app.add_directive('tikz', TikzDirective)
-    app.add_config_value('tikz_latex_preamble', '', 'html')
-    app.add_config_value('tikz_tikzlibraries', '', 'html')
+    app.add_config_value('tikz_latex_preamble', '', 'env')
+    app.add_config_value('tikz_tikzlibraries', '', 'env')
     app.add_config_value('tikz_transparent', True, 'html')
+    app.add_config_value('tikz_includegraphics_path', '', 'env')
 
     # fallback to another value depending what is on the system
     suite = 'pdf2svg'
